Summary
=============================
> Direct access read at EOF

## Root Cause Analysis

POSIX specification of [read](http://pubs.opengroup.org/onlinepubs/9699919799/) says:

> No data transfer shall occur past the current end-of-file. *If the starting
> position is at or after the end-of-file, 0 shall be returned*. If the file
> refers to a device special file, the result of subsequent read()
> requests is implementation-defined.

ext4 does not seem to check the case when the starting position is at or after EOF
for direct access read. In the test program included, read fails with EINVAL
due to [misalignment](http://elixir.free-electrons.com/linux/v4.13-rc7/source/fs/direct-io.c#L1137),
instead of returning 0 as indicated in the specification.

## POC

* Run ./min in a ext4 filesystem
```
$ cd ext4-fs/
$ g++ -o min min.cpp
$ echo "hello world" > file0
$ ./min
mmap()=536870912 errno=0
open()=3 errno=0
lseek()=12 errno=0
read()=-1 errno=22
```

* Run ./min in a xfs filesystem
```
$ cd xfs-fs/
$ g++ -o min min.cpp
$ echo "hello world" > file0
$ ./min
mmap()=536870912 errno=0
open()=3 errno=0
lseek()=12 errno=0
read()=0 errno=0
```


## Analysis

* Other filesystems such as XFS use function `iomap_dio_rw` for direct access
read which checks the starting position of the read first.

[fs/iomap.c:iomap_dio_rw](http://elixir.free-electrons.com/linux/v4.13-rc7/source/fs/iomap.c#L968)
```
958             dio->error = 0;
959             dio->flags = 0;
...

968             if (iov_iter_rw(iter) == READ) {
969                     if (pos >= dio->i_size)
970                             goto out_free_dio; }
```



