Summary
=========================
> Inconsistent error code (lseek)

## Root Cause Analysis

* Ext4 does not seem to check if the file size equals to 0 for syscall lseek,
while other filesystems such as xfs/btrfs/f2fs perform this check at the very beginning.
Therefore, in many cases lseek in ext4 returns different error codes from in
other popular filesystems.

* [btrfs](http://elixir.free-electrons.com/linux/v4.13-rc6/source/fs/btrfs/file.c#L3020)
```
http://elixir.free-electrons.com/linux/v4.13-rc6/source/fs/btrfs/file.c#L3020
3020    if (inode->i_size == 0)
3021                return -ENXIO;

```

* [f2fs](http://elixir.free-electrons.com/linux/v4.13-rc6/source/fs/iomap.c#L614)
```
613     /* Nothing to be found before or beyond the end of the file. */
614     if (offset < 0 || offset >= size)
615         return -ENXIO;
```

##  POC

* Run in a ext4 filesystem

```
$ cd ext4-fs/
$ g++ -o errno_117 errno_117.cpp
$ ./errno_117
create()=3, errno=0
lseek()=-1, errno=117

$ rm -rf file0
$ g++ -o errno_22 errno_22.cpp
$ ./errno_22
create()=3 errno=0
lseek()=-1 errno=22
```

* Run in another filesystem (xfs/f2fs)

```
$ cd xfs-fs/
$ g++ -o errno_117 errno_117.cpp
$ ./errno_117
create()=3, errno=0
lseek()=-1, errno=6

$ rm -rf file0
$ g++ -o errno_22 errno_22.cpp
$ ./errno_22
create()=3 errno=0
lseek()=-1 errno=6
```

### Analysis

* For errno_117.cpp, lseek in ext4 fails from [here](http://elixir.free-electrons.com/linux/v4.13-rc6/source/fs/ext4/inode.c#L510)

```
510     /* We can handle the block number less than EXT_MAX_BLOCKS */
511    if (unlikely(map->m_lblk >= EXT_MAX_BLOCKS))
512                return -EFSCORRUPTED;
```
```
3126 /*
3127 * Maximum number of logical blocks in a file; ext4_extent's ee_block is
3128 * __le32.
3129 */
3130 #define EXT_MAX_BLOCKS  0xffffffff
```
